/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template provides access to some LEDs and buttons of the board.
*
*******************************************************************************/

#include "am_bsp.h"
#include "am_util.h"

#include "ewconfig.h"
#include "ew_bsp_inout.h"

static TButtonCallback ButtonCallback = 0;

//*****************************************************************************
// GPIO ISR
//*****************************************************************************
void am_gpio0_001f_isr(void)
{
  uint32_t ui32IntStatus;
  static uint32_t pinState = 0;

  /* delay for button debounce */
  am_hal_delay_us(20000);

  /* clear the GPIO Interrupt (write to clear) */
  AM_CRITICAL_BEGIN
  am_hal_gpio_interrupt_irq_status_get( GPIO0_001F_IRQn, true, &ui32IntStatus );
  am_hal_gpio_interrupt_irq_clear( GPIO0_001F_IRQn, ui32IntStatus );
  AM_CRITICAL_END

  /* toggle the pin status */
  pinState = !pinState;

  /* configure the button pin to use rising or falling edge -
    (AM_HAL_GPIO_PIN_INTDIR_BOTH seems not to work for some reasons...) */
  if ( pinState )
    g_AM_BSP_GPIO_BUTTON0.GP.cfg_b.eIntDir = AM_HAL_GPIO_PIN_INTDIR_LO2HI;
  else
    g_AM_BSP_GPIO_BUTTON0.GP.cfg_b.eIntDir = AM_HAL_GPIO_PIN_INTDIR_HI2LO;
  am_hal_gpio_pinconfig( AM_BSP_GPIO_BUTTON0, g_AM_BSP_GPIO_BUTTON0 );

  /* call the callback */
  if ( ButtonCallback )
    ButtonCallback( pinState );
}

/*******************************************************************************
* FUNCTION:
*   EwBspInOutInitButton
*
* DESCRIPTION:
*   Configures one hardware button of the board used for demo applications.
*
* ARGUMENTS:
*   aButtonCallback - The button callback.
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutInitButton( TButtonCallback aButtonCallback )
{
  ButtonCallback = aButtonCallback;

  uint32_t ui32IntStatus;
  uint32_t ui32BUTTON0GpioNum = AM_BSP_GPIO_BUTTON0;

  /* configure the button pin */
  g_AM_BSP_GPIO_BUTTON0.GP.cfg_b.eIntDir = AM_HAL_GPIO_PIN_INTDIR_BOTH;
  am_hal_gpio_pinconfig( AM_BSP_GPIO_BUTTON0, g_AM_BSP_GPIO_BUTTON0 );

  /* clear the GPIO Interrupt (write to clear) */
  AM_CRITICAL_BEGIN
  am_hal_gpio_interrupt_irq_status_get( GPIO0_001F_IRQn, false, &ui32IntStatus );
  am_hal_gpio_interrupt_irq_clear( GPIO0_001F_IRQn, ui32IntStatus );
  AM_CRITICAL_END

  /* enable the GPIO/button interrupt */
  am_hal_gpio_interrupt_control( AM_HAL_GPIO_INT_CHANNEL_0,
    AM_HAL_GPIO_INT_CTRL_INDV_ENABLE, (void*)&ui32BUTTON0GpioNum );
  NVIC_SetPriority( GPIO0_001F_IRQn, AM_IRQ_PRIORITY_DEFAULT );
  NVIC_EnableIRQ( GPIO0_001F_IRQn );
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutInitLed
*
* DESCRIPTION:
*   Configures one LED of the board used for demo applications.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutInitLed( void )
{
  am_devices_led_init( am_bsp_psLEDs );
  am_devices_led_off( am_bsp_psLEDs, 0 );
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutLedOn
*
* DESCRIPTION:
*   Switch LED on (used for demo applications).
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutLedOn( void )
{
  am_devices_led_on( am_bsp_psLEDs, 0 );
}


/*******************************************************************************
* FUNCTION:
*   EwBspInOutLedOff
*
* DESCRIPTION:
*   Switch LED off (used for demo applications).
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspInOutLedOff( void )
{
  am_devices_led_off( am_bsp_psLEDs, 0 );
}


/* msy */
