/*******************************************************************************
*
* E M B E D D E D   W I Z A R D   P R O J E C T
*
*                                                Copyright (c) TARA Systems GmbH
*                                    written by Paul Banach and Manfred Schweyer
*
********************************************************************************
*
* This software is delivered "as is" and shows the usage of other software
* components. It is provided as an example software which is intended to be
* modified and extended according to particular requirements.
*
* TARA Systems hereby disclaims all warranties and conditions with regard to the
* software, including all implied warranties and conditions of merchantability
* and non-infringement of any third party IPR or other rights which may result
* from the use or the inability to use the software.
*
********************************************************************************
*
* DESCRIPTION:
*   This file is part of the interface (glue layer) between an Embedded Wizard
*   generated UI application and the board support package (BSP) of a dedicated
*   target.
*   This template is responsible to configurate the entire system (CPU clock,
*   memory, qspi, etc).
*
*******************************************************************************/

#include "am_mcu_apollo.h"
#include "am_bsp.h"
#include "am_util.h"
#include "am_devices_mspi_psram_aps12808l.h"

#include "ewconfig.h"
#include "ew_bsp_system.h"
#include "ew_bsp_clock.h"

#if EW_USE_PSRAM == 1

  #define ENABLE_XIPMM
  #define MSPI_PSRAM_MODULE              0

  #if (MSPI_PSRAM_MODULE == 0)
  #define MSPI_XIP_BASE_ADDRESS 0x14000000
  #elif (MSPI_PSRAM_MODULE == 1)
  #define MSPI_XIP_BASE_ADDRESS 0x18000000
  #elif (MSPI_PSRAM_MODULE == 2)
  #define MSPI_XIP_BASE_ADDRESS 0x1C000000
  #endif // #if (MSPI_PSRAM_MODULE == 0)

  #define MSPI_XIPMM_BASE_ADDRESS MSPI_XIP_BASE_ADDRESS

  #define DDR_PSRAM_TARGET_ADDRESS 0

  AM_SHARED_RW uint32_t        g_ui32DMATCBBuffer[2560];
  void            *g_pPSRAMHandle;
  void            *g_pMSPIHandle;

  am_devices_mspi_psram_config_t MSPI_PSRAM_OctalCE1MSPIConfig =
  {
      .eDeviceConfig            = AM_BSP_MSPI_PSRAM_MODULE_OCTAL_DDR_CE,
      .eClockFreq               = AM_HAL_MSPI_CLK_96MHZ,
      .ui32NBTxnBufLength       = sizeof(g_ui32DMATCBBuffer) / sizeof(uint32_t),
      .pNBTxnBuf                = g_ui32DMATCBBuffer,
      .ui32ScramblingStartAddr  = 0,
      .ui32ScramblingEndAddr    = 0,
  };
  //! MSPI interrupts.
  static const IRQn_Type mspi_interrupts[] =
  {
      MSPI0_IRQn,
  #if defined(AM_PART_APOLLO3P) || defined(AM_PART_APOLLO4) || defined(AM_PART_APOLLO4B)
      MSPI1_IRQn,
      MSPI2_IRQn,
  #endif
  };

  //
  // Take over the interrupt handler for whichever MSPI we're using.
  //
  #define psram_mspi_isr                                                          \
      am_mspi_isr1(MSPI_PSRAM_MODULE)
  #define am_mspi_isr1(n)                                                        \
      am_mspi_isr(n)
  #define am_mspi_isr(n)                                                         \
      am_mspi ## n ## _isr

  //*****************************************************************************
  //
  // MSPI ISRs.
  //
  //*****************************************************************************
  void psram_mspi_isr(void)
  {
      uint32_t      ui32Status;

      am_hal_mspi_interrupt_status_get(g_pMSPIHandle, &ui32Status, false);

      am_hal_mspi_interrupt_clear(g_pMSPIHandle, ui32Status);

      am_hal_mspi_interrupt_service(g_pMSPIHandle, ui32Status);
  }

#endif


/*******************************************************************************
* FUNCTION:
*   EwBspSystemInit
*
* DESCRIPTION:
*   The function EwBspSystemInit initializes the system components.
*   (CPU clock, memory, qspi, ...)
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspSystemInit( void )
{
  uint32_t ui32Status;

  am_bsp_external_pwr_on();
  am_util_delay_ms(100);
  am_bsp_low_power_init();

  am_hal_cachectrl_config(&am_hal_cachectrl_defaults);
  am_hal_cachectrl_enable();


  #if EW_USE_PSRAM == 1

    // Configure the MSPI and PSRAM Device.
    //
    ui32Status = am_devices_mspi_psram_aps12808l_ddr_init(MSPI_PSRAM_MODULE, &MSPI_PSRAM_OctalCE1MSPIConfig, &g_pPSRAMHandle, &g_pMSPIHandle);
    if (AM_DEVICES_MSPI_PSRAM_STATUS_SUCCESS != ui32Status)
    {
        am_util_stdio_printf("Failed to configure the MSPI and PSRAM Device correctly!\n");
    }
    NVIC_SetPriority(mspi_interrupts[MSPI_PSRAM_MODULE], AM_IRQ_PRIORITY_DEFAULT);
    NVIC_EnableIRQ(mspi_interrupts[MSPI_PSRAM_MODULE]);

    //
    // Enable XIP mode.
    //
    ui32Status = am_devices_mspi_psram_aps12808l_ddr_enable_xip(g_pPSRAMHandle);
    if (AM_DEVICES_MSPI_PSRAM_STATUS_SUCCESS == ui32Status)
    {
        am_util_stdio_printf("Enable XIP mode in the MSPI!\n");
    }
    else
    {
        am_util_stdio_printf("Failed to enable XIP mode in the MSPI!\n");
    }

  #endif

  //
  // If you want to change display configurations in example, please change them here as below.
  //
  // g_eDispType = RM67162_DSI;
  // g_sDsiCfg.ui8NumLanes = 1;
  // g_sDsiCfg.eDbiWidth = AM_HAL_DSI_DBI_WIDTH_8;
  // g_sDsiCfg.eDsiFreq = AM_HAL_DSI_FREQ_TRIM_X12;
  if ((g_sDispCfg[g_eDispType].eInterface == IF_DSI) || (g_sDispCfg[g_eDispType].bUseDPHYPLL == true))
  {
      //
      // VDD18 control callback function
      //
      am_hal_dsi_register_external_vdd18_callback(am_bsp_external_vdd18_switch);
      //
      // Enable DSI power and configure DSI clock.
      //
      am_hal_dsi_init();
  }
  else
  {
      am_hal_clkgen_control(AM_HAL_CLKGEN_CONTROL_DISPCLKSEL_HFRC96, NULL);
      am_hal_clkgen_control(AM_HAL_CLKGEN_CONTROL_DCCLK_ENABLE, NULL);
  }
  am_hal_pwrctrl_periph_enable(AM_HAL_PWRCTRL_PERIPH_GFX);
  am_hal_pwrctrl_periph_enable(AM_HAL_PWRCTRL_PERIPH_DISP);

  //
  // Global interrupt enable
  //
  am_hal_interrupt_master_enable();

  am_hal_timer_config_t sTimerConfig;

  ui32Status = am_hal_timer_default_config_set(&sTimerConfig);
  if (AM_HAL_STATUS_SUCCESS != ui32Status)
  {
      am_util_stdio_printf("Failed to initialize a timer configuration structure with default values!\n");
  }
  sTimerConfig.eInputClock = AM_HAL_TIMER_CLOCK_HFRC_DIV16;
  sTimerConfig.eFunction = AM_HAL_TIMER_FN_REPEATPATTERN;
  ui32Status = am_hal_timer_config(0, &sTimerConfig);
  if (AM_HAL_STATUS_SUCCESS != ui32Status)
  {
      am_util_stdio_printf("Failed to configure a timer!\n");
  }
  ui32Status = am_hal_timer_start(0);
  if (AM_HAL_STATUS_SUCCESS != ui32Status)
  {
      am_util_stdio_printf("Failed to start a timer!\n");
  }
}


/*******************************************************************************
* FUNCTION:
*   EwBspSystemDone
*
* DESCRIPTION:
*   The function EwBspSystemDone terminates the system components.
*
* ARGUMENTS:
*   None
*
* RETURN VALUE:
*   None
*
*******************************************************************************/
void EwBspSystemDone( void )
{
}


/* msy, mli */
